#include "brock.h"

#define _CRT_SECURE_NO_WARNINGS 1

#include <windows.h>
#include <stdio.h>
#include <time.h>
//#include <stddef.h>
//#include <string.h>
//#include <stdarg.h>
//#include <stdlib.h>		

#if defined(_DEBUG) && BROCK_DEBUG
#define BROCK_PARANOID	1
#else
#define BROCK_PARANOID	0
#endif


#if BROCK_DEBUG || BROCK_PROFILE
#include "generator.h" // for print() functions
#endif


typedef unsigned	char		uint8;
typedef signed		char		sint8;
typedef unsigned	short		uint16;
typedef signed		short		sint16;
typedef unsigned	int			uint32;
typedef signed		int			sint32;
typedef unsigned	long int	uint64;
typedef signed		long int	sint64;
typedef				float		float32;
typedef				double		float64;


//typedef unsigned	char		val_t;
typedef unsigned	int			val_t;



void DebugMessage( char * format, ... )
{
	char buffer[1024];
	buffer[0] = 0;
	buffer[1023] = 0;

	va_list arg_list;
	va_start(arg_list, format);
	vsnprintf( buffer, sizeof(buffer), format, arg_list );
	va_end(arg_list);

	buffer[1023] = 0;

#if BROCK_DEBUG || BROCK_PROFILE
	printf(buffer);
#endif
	OutputDebugStringA(buffer);
}

#if BROCK_DEBUG
#define FO_REALZ(exp, msg)	if (!(exp)){ DebugMessage("Error %s, line %d: %s\n %s\n", __FILE__, __LINE__, #exp, msg); DebugBreak(); } 
#else
#define FO_REALZ(exp, msg)	
#endif

#if BROCK_PARANOID
#define PARANOID(exp, msg)	if (!(exp)){ DebugMessage("Error %s, line %d: %s\n %s\n", __FILE__, __LINE__, #exp, msg); DebugBreak(); } 
#else
#define PARANOID(exp, msg)	
#endif



const uint32 NUM_ROWS	= 9;
const uint32 NUM_COLS	= 9;
const uint32 CELL_COUNT	= 81;
const uint32 BOX_COUNT	= 9;

const val_t	MAX_SET_VALUE = 9 + 8 + 7 + 6 + 5 + 4 + 3 + 2 + 1;	// the max sum of one row, column, or box;

const uint32 POINTS_PER_VALUE	= 1;
const uint32 POINTS_PER_ROW		= 10;
const uint32 POINTS_PER_COL		= 10;
const uint32 POINTS_PER_BOX		= 10;

const uint32 BIT_0	= 1 << 0;	// 1st bit, 1
const uint32 BIT_1	= 1 << 1;	// 2
const uint32 BIT_2	= 1 << 2;	// 4
const uint32 BIT_3	= 1 << 3;	// 8
const uint32 BIT_4	= 1 << 4;	// 16
const uint32 BIT_5	= 1 << 5;	// 32
const uint32 BIT_6	= 1 << 6;	// 64
const uint32 BIT_7	= 1 << 7;	// 128
const uint32 BIT_8	= 1 << 8;	// 256
const uint32 BIT_9	= 1 << 9;	// 512
const uint32 BIT_10	= 1 << 10;	// 1024
const uint32 BIT_11	= 1 << 11;	// 2048
const uint32 BIT_12	= 1 << 12;	// 4096
const uint32 BIT_13	= 1 << 13;	// 8192
const uint32 BIT_14	= 1 << 14;	// 16384
const uint32 BIT_15	= 1 << 15;	// 16th bit, 32768


class Timer
{
public:
	Timer(): m_bStarted(false)
	{
		::QueryPerformanceFrequency(&m_Frequency);
	}

	void Start()
	{
		m_bStarted = true;
		::QueryPerformanceCounter(&m_StartTime);		
	}
	

	float64 Elapsed() const
	{
		if ( !m_bStarted) 
			return 0.0;

		LARGE_INTEGER EndTime;
		::QueryPerformanceCounter(&EndTime);

		LONGLONG diff = EndTime.QuadPart - m_StartTime.QuadPart;
	
		return ((float64)diff) / (float64(m_Frequency.QuadPart));
	}

	float64 Restart()
	{
		const float64 fElapsed = Elapsed();

		Start();

		return fElapsed;
	}


private:

	LARGE_INTEGER m_StartTime;
	LARGE_INTEGER m_Frequency;
	bool m_bStarted;

};

// cheesy random number generator, based off of something I found on a web page. 
// I need to replace this... 
class Random
{
public:

	Random()
	{
		Reseed( (uint32)time(NULL) );
	}

	void Reseed( uint32 seed )
	{
		::srand( seed );
	}

	// Generates a pseudo-random integer between 0 and RAND_MAX 
	sint32 GetInt()
	{
		return ::rand();
	}

	// Generates a pseudo-random integer between 0 and max
	sint32 GetInt( const int max_val )
	{
		return (sint32) ( (float)max_val * ( (float)::rand() /((float)RAND_MAX+1.0f)) );
	}

	// Generates a pseudo-random integer between min and max INCLUSIVE
	sint32 GetInt( const int min_val, const int max_val )
	{
		if ( min_val > max_val )		
			return (int)( max_val + ( (min_val-max_val+1) * (::rand()/(RAND_MAX+1.0f)) ) );
		else							
			return (int)( min_val + ( (max_val-min_val+1 ) * (::rand()/(RAND_MAX+1.0f)) ) );
	}
	

};



//static const uint32 OCCUPANT_INDICES_MASK	=	BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 | BIT_8 | BIT_9;
//static const uint32 OCCUPANT_COUNT_MASK	=	BIT_10 | BIT_11 | BIT_12 | BIT_13 | BIT_14 | BIT_15;
//static const uint32 OCCUPANT_COUNT_SHIFT	=	10;
static const uint32 OCCUPANT_INDICES_MASK	=	0x0000FFFF;
static const uint32 OCCUPANT_COUNT_MASK		=	0xFFFF0000;
static const uint32 OCCUPANT_COUNT_SHIFT	=	16;


class Sudoku_Board
{
public:
	Sudoku_Board();
	
	void ToCharArray( char * board );

	inline void Reset();
	inline void CopyFrom( const Sudoku_Board *pOther);

	static inline void GetBoxIndices(	const uint32 nRow, const uint32 nCol,
										uint32 &nBoxRowMin, uint32 &nBoxRowMax,
										uint32 &nBoxColMin, uint32 &nBoxColMax );
	
	inline uint32 GetScore( void ) const;
	inline val_t GetCellValue( const uint32 nRow, const uint32 nCol ) const;
	inline bool IsCellOccupied( const uint32 nRow, const uint32 nCol ) const;
	inline uint32 GetRowCount( const uint32 nRow ) const;
	inline uint32 GetColCount( const uint32 nCol ) const;	
	inline uint32 GetBoxCount( const uint32 nRow, const uint32 nCol ) const;
	inline bool IsRowFull( const uint32 nRow ) const;
	inline bool IsColFull( const uint32 nCol ) const;
	inline bool IsBoxFull( const uint32 nRow, const uint32 nCol ) const;
	inline bool IsSolved() const;
	inline bool IsUnsolveable() const;
	inline bool IsValueAvailable( const val_t nVal ) const;

	inline bool IsValueInRow( const val_t nVal, const uint32 nRow ) const;
	inline bool IsValueInCol( const val_t nVal, const uint32 nCol ) const;
	inline bool IsValueInBox( const val_t nVal, const uint32 nRow, const uint32 nCol ) const;

	inline bool IsValueValid( const val_t nVal, const uint32 nRow, const uint32 nCol ) const;

	// Set the value at the specified position, and returns the new score
	inline uint32 SetCellValue( const val_t nVal, const uint32 nRow, const uint32 nCol );

	inline uint32 ClearCellValue( const uint32 nRow, const uint32 nCol );

	// This will check all, rows, columns, and boxes for single missing entries and
	// fills them in. Returns the new score.
	uint32 FindTrivialSolutions();

	uint32 FindTrivialSolutions(const uint32 nRow, const uint32 nCol);

	bool CheckValidity() const;

private:
	val_t	m_Board[NUM_ROWS][NUM_COLS];
	uint32	m_nScore; // the "score" for this board
	uint32	m_nEmptyCells;

	uint32	m_RowCache[NUM_ROWS];	// bits 0..9 track which indices are filled in
	uint32	m_ColCache[NUM_COLS];	// bits 10..16 track total # of items 
	uint32	m_nValueCache[10];		// tracks # of each value remaining (indexed so that '1' is 1, etc, 0 is wasted)

	
	bool	m_bPooched;
};

Sudoku_Board::Sudoku_Board()
{
	
}

void Sudoku_Board::ToCharArray( char * board )
{
	for ( uint32 r = 0; r < NUM_ROWS; ++r )
	{
		for ( uint32 c = 0; c < NUM_COLS; ++c )
		{
			char s = (char) m_Board[r][c];
			board[c+(r*9)] = s;
		}
	}
}

void Sudoku_Board::Reset()
{
	// TODO: profile... 
#if 1
	memset( m_Board, 0, sizeof(m_Board) );
	memset( m_RowCache, 0, sizeof(m_RowCache) );
	memset( m_ColCache, 0, sizeof(m_ColCache) );
#else
	for ( uint32 r = 0; r < NUM_ROWS; r++ )
	{
		for ( uint32 c = 0; c <  NUM_COLS; c++ )
		{
			m_Board[r][c] = 0;
		}
	}
	for ( uint32 r = 0; r < NUM_ROWS; r++ )
	{
		m_RowCache[0] = 0;
		m_ColCache[0] = 0;
	}
#endif

	m_nScore = 0;
	m_nEmptyCells = 81;

	m_nValueCache[0] = 0;
	for ( int i = 1; i <= 9; ++i )
	{
		m_nValueCache[i] = 9;
	}

	m_bPooched = false;
}

inline void Sudoku_Board::CopyFrom( const Sudoku_Board *pOther)
{
	FO_REALZ( this != pOther, "Board being asked to copy onto itself!" );

	memcpy( this, pOther, sizeof(Sudoku_Board) );
}


inline void Sudoku_Board::GetBoxIndices( const uint32 nRow, const uint32 nCol,
										 uint32 &nBoxRowMin, uint32 &nBoxRowMax,
										 uint32 &nBoxColMin, uint32 &nBoxColMax ) 
{
	FO_REALZ( nRow <=8, "Row Index out of bounds!" );
	FO_REALZ( nCol <=8, "Col Index out of bounds!" );

	nBoxRowMin = (nRow / 3) *3;
	nBoxRowMax = nBoxRowMin + 3;		  
	nBoxColMin = (nCol / 3) *3;
	nBoxColMax = nBoxColMin + 3;

	FO_REALZ( nBoxRowMin <= 6, "Row Index out of bounds!" );
	FO_REALZ( nBoxRowMax <= 9, "Row Index out of bounds!" );
	FO_REALZ( nBoxColMin <= 6, "Col Index out of bounds!" );
	FO_REALZ( nBoxColMax <= 9, "Col Index out of bounds!" );
}

inline uint32 Sudoku_Board::GetScore() const
{
	return m_nScore;
}

inline val_t Sudoku_Board::GetCellValue(const uint32 nRow, const uint32 nCol) const
{
	FO_REALZ( nRow <=8, "Row Index out of bounds!" );
	FO_REALZ( nCol <=8, "Col Index out of bounds!" );

	const val_t nValue = m_Board[nRow][nCol];

	//FO_REALZ( nValue > 0, "Cell Value out of bounds!" );
	FO_REALZ( nValue <= 9, "Cell Value out of bounds!" );

	return nValue;
}


inline bool Sudoku_Board::IsCellOccupied( const uint32 nRow, const uint32 nCol ) const
{
	FO_REALZ( nRow <=8, "Row Index out of bounds!" );
	FO_REALZ( nCol <=8, "Col Index out of bounds!" );

	if ( 0 == m_Board[nRow][nCol] )
		return false;

	return true;
}



inline uint32 Sudoku_Board::GetRowCount( const uint32 nRow ) const
{
	FO_REALZ( nRow <=8, "Row Index out of bounds!" );

	const uint32 nRowUnmasked = m_RowCache[nRow] & OCCUPANT_COUNT_MASK;
	const uint32 nRowCount = nRowUnmasked >> OCCUPANT_COUNT_SHIFT;

	FO_REALZ( nRowCount <= 9, "Row Count out of bounds!" );

	return nRowCount;
}

inline uint32 Sudoku_Board::GetColCount( const uint32 nCol ) const
{
	FO_REALZ( nCol <=8, "Col Index out of bounds!" );

	const uint32 nColUnmasked = m_ColCache[nCol] & OCCUPANT_COUNT_MASK;
	const uint32 nColCount = nColUnmasked >> OCCUPANT_COUNT_SHIFT;

	FO_REALZ( nColCount <= 9, "Column Count out of bounds!" );

	return nColCount;
}

inline uint32 Sudoku_Board::GetBoxCount( const uint32 nRow, const uint32 nCol ) const
{
	uint32 nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax;
	
	GetBoxIndices( nRow, nCol, nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax );
	
	uint32 nBoxCount = 0;

	for ( uint32 nBoxRow = nBoxRowMin; nBoxRow < nBoxRowMax; ++nBoxRow )
	{
		for ( uint32 nBoxCol = nBoxColMin; nBoxCol < nBoxColMax; ++nBoxCol )
		{
			nBoxCount += ( 0 != m_Board[nBoxRow][nBoxCol] );
		}
	}

	FO_REALZ( nBoxCount <= 9, "Box Count out of bounds!" );

	return nBoxCount;
}

inline bool Sudoku_Board::IsRowFull( const uint32 nRow ) const
{
	const uint32 nRowUnmasked = m_RowCache[nRow] & OCCUPANT_INDICES_MASK;
	return ( nRowUnmasked == OCCUPANT_INDICES_MASK );
}

inline bool Sudoku_Board::IsColFull( const uint32 nCol ) const
{
	const uint32 nColUnmasked = m_ColCache[nCol] & OCCUPANT_INDICES_MASK;
	return ( nColUnmasked == OCCUPANT_INDICES_MASK );
}

inline bool Sudoku_Board::IsBoxFull( const uint32 nRow, const uint32 nCol ) const
{
	const uint32 nCount = GetBoxCount( nRow, nCol );
	return ( nCount == 9 );
}

inline bool Sudoku_Board::IsSolved() const
{
	return m_nEmptyCells == 0;
}


inline bool Sudoku_Board::IsUnsolveable() const
{
	if ( m_bPooched )
		return false;
	return true;
}

inline bool Sudoku_Board::IsValueAvailable( const val_t nVal ) const
{
	if ( m_nValueCache[nVal] == 0 )
		return false;
	return true;
}


inline bool Sudoku_Board::IsValueInRow( const val_t nVal, const uint32 nRow ) const
{
	// TODO: unroll by hand? 
	for ( uint32 nCol = 0; nCol < NUM_COLS; ++nCol )
	{
		if ( nVal == m_Board[nRow][nCol] )
			return true;
	}

	return false;
}

inline bool Sudoku_Board::IsValueInCol( const val_t nVal, const uint32 nCol ) const
{
	// TODO: unroll by hand? 
	for ( uint32 nRow = 0; nRow < NUM_ROWS; ++nRow )
	{
		if ( nVal == m_Board[nRow][nCol] )
			return true;
	}

	return false;
}

inline bool Sudoku_Board::IsValueInBox( const val_t nVal, const uint32 nRow, const uint32 nCol ) const
{
	FO_REALZ( nRow <=8, "Row Index out of bounds!" );
	FO_REALZ( nCol <=8, "Col Index out of bounds!" );

	uint32 nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax;	
	GetBoxIndices( nRow, nCol, nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax );

	for ( uint32 nBoxRow = nBoxRowMin; nBoxRow < nBoxRowMax; ++nBoxRow )
	{
		for ( uint32 nBoxCol = nBoxColMin; nBoxCol < nBoxColMax; ++nBoxCol )
		{
			if ( nVal == m_Board[nBoxRow][nBoxCol] )
				return true;
		}
	}

	return false;
}

inline bool Sudoku_Board::IsValueValid( const val_t nVal, const uint32 nRow, const uint32 nCol ) const
{
	FO_REALZ( nRow <=8, "Row Index out of bounds!" );
	FO_REALZ( nCol <=8, "Col Index out of bounds!" );

	// Fast and generally useful
	if ( IsCellOccupied( nRow, nCol ) )
		return false;

	// Fast, but only culls late in search
	if ( 0 == m_nValueCache[nVal] )
		return false;

	// not too bad...
	if ( IsValueInRow(nVal, nRow) )
		return false;

	// not too bad...
	if ( IsValueInCol(nVal, nCol) )
		return false;

	// Slowest...
	if ( IsValueInBox(nVal, nRow, nCol) )
		return false;


	return true;
}

// returns the new score
inline uint32 Sudoku_Board::SetCellValue( const val_t nVal, const uint32 nRow, const uint32 nCol )
{
#if 0
	if ( !IsValueValid(nVal, nRow, nCol) )
	{
		char board[100]; 
		memset(board, 0, sizeof(board) );
		
		DebugMessage( "Board being asked to set invalid value %d at %d, %d\n", nVal, nRow, nCol );
		Generator::print( board );
	}
#endif

	FO_REALZ( IsValueValid(nVal, nRow, nCol), "Being asked to set invalid value!"  );
	FO_REALZ( nVal > 0, "Being asked to set value out of range!" );
	FO_REALZ( nVal <= 9, "Being asked to set value out of range!" );


	// set the value in the board
	m_Board[nRow][nCol] = nVal;
	--m_nEmptyCells;


	// update the row cache
	const uint32 nNewRowCount			= GetRowCount(nRow) + 1;
	const uint32 nNewRowCountShifted	= nNewRowCount << OCCUPANT_COUNT_SHIFT;
	const uint32 nRowIndices			= m_RowCache[nRow] & OCCUPANT_INDICES_MASK;
	const uint32 nNewRowIndices			= nRowIndices | (1 << nCol);
	m_RowCache[nRow] = nNewRowIndices | nNewRowCountShifted;
	

	// update the column cache	
	const uint32 nNewColCount			= GetColCount(nCol) + 1;
	const uint32 nNewColCountShifted	= nNewColCount << OCCUPANT_COUNT_SHIFT;
	const uint32 nColIndices			= m_ColCache[nCol] & OCCUPANT_INDICES_MASK;
	const uint32 nNewColIndices			= nColIndices | (1 << nRow);
	m_ColCache[nCol] = nNewColIndices | nNewColCountShifted;
	

	// update the # of times we can use this value 
	--m_nValueCache[nVal];


	// now award some points...

	// we get some points just for filling in a value. yay!
	m_nScore += POINTS_PER_VALUE;

	// and more points for filling up a row / column
	m_nScore += POINTS_PER_ROW * IsRowFull(nRow);
	m_nScore += POINTS_PER_COL * IsColFull(nCol);

	// and some more for filling in a box. joy!
	m_nScore += POINTS_PER_BOX * IsBoxFull(nRow, nCol);


	PARANOID( CheckValidity(), "Board invalid in SetCellValue!" )

	return m_nScore;
}

uint32 Sudoku_Board::ClearCellValue( const uint32 nRow, const uint32 nCol )
{
	const uint32 nOldScore = m_nScore;
	const val_t nOldValue = m_Board[nRow][nCol];

	FO_REALZ( nOldValue != 0, "Being asked to clear a cleared cell!" );



	// Remove points
	m_nScore -= POINTS_PER_VALUE;
	m_nScore -= POINTS_PER_ROW * IsRowFull(nRow);
	m_nScore -= POINTS_PER_COL * IsColFull(nCol);
	m_nScore -= POINTS_PER_BOX * IsBoxFull(nRow, nCol);

	// update the # of times we can use this value 
	++m_nValueCache[nOldValue];


	// update the row cache
	const uint32 nNewRowCount			= GetRowCount(nRow) - 1;
	const uint32 nNewRowCountShifted	= nNewRowCount << OCCUPANT_COUNT_SHIFT;
	const uint32 nRowIndices			= m_RowCache[nRow] & OCCUPANT_INDICES_MASK;
	const uint32 nNewRowIndices			= nRowIndices & ~(1 << nCol);
	m_RowCache[nRow] = nNewRowIndices | nNewRowCountShifted;


	// update the column cache	
	const uint32 nNewColCount			= GetColCount(nCol) - 1;
	const uint32 nNewColCountShifted	= nNewColCount << OCCUPANT_COUNT_SHIFT;
	const uint32 nColIndices			= m_ColCache[nCol] & OCCUPANT_INDICES_MASK;
	const uint32 nNewColIndices			= nColIndices & ~(1 << nRow);
	m_ColCache[nCol] = nNewColIndices | nNewColCountShifted;


	// clear the value in the board
	m_Board[nRow][nCol] = 0;
	++m_nEmptyCells;

	PARANOID( CheckValidity(), "Board invalid in ClearCellValue!" )
	FO_REALZ( nOldScore > m_nScore, "Board incorrectly scored in ClearCellValue" )


	return m_nScore;
}

uint32 Sudoku_Board::FindTrivialSolutions()
{
	const uint32 nOldScore = m_nScore;

	// Check for holes in rows
	for ( uint32 nRow = 0; nRow < NUM_ROWS; nRow++ )
	{
		if ( 8 != GetRowCount(nRow) )
			continue;

		// ah ha! This row is only missing 1 value, so fill it in!

		uint32 nValuesUsed = MAX_SET_VALUE;
		uint32 nEmptyCol = 99;

		for ( uint32 nCol = 0; nCol < NUM_COLS; nCol++ )
		{
			const val_t nValue = m_Board[nRow][nCol];
			if ( nValue == 0 )
				nEmptyCol = nCol;
			else
				nValuesUsed -= nValue;
		}

		if ( IsValueValid( nValuesUsed, nRow, nEmptyCol ) )
		{
			SetCellValue( nValuesUsed, nRow, nEmptyCol ); 
		}
		else
		{
			// board is a dud
			m_bPooched = true;
		}
	}

	// Check for holes in columns
	for ( uint32 nCol = 0; nCol < NUM_COLS; nCol++ )
	{
		if ( 8 != GetColCount(nCol) )
			continue;

		// ah ha! This column is only missing 1 value, so fill it in!

		uint32 nValuesUsed = MAX_SET_VALUE;
		uint32 nEmptyRow = 99;
		
		for ( uint32 nRow = 0; nRow < NUM_ROWS; nRow++ )
		{
			const val_t nValue = m_Board[nRow][nCol];
			if ( nValue == 0 )
				nEmptyRow = nRow;
			else
				nValuesUsed -= nValue;
		}

		if ( IsValueValid( nValuesUsed, nEmptyRow, nCol ) )
		{
			SetCellValue( nValuesUsed, nEmptyRow, nCol );	
		}
		else
		{
			// board is a dud
			m_bPooched = true;
		}
	}

	// Check for holes in boxes
	for ( uint32 nRow = 0; nRow < NUM_ROWS; nRow+= 3 )
	{
		for ( uint32 nCol = 0; nCol < NUM_COLS; nCol+= 3 )	
		{
			if ( 8 != GetBoxCount( nRow, nCol ) )
				continue;

			uint32 nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax;	
			GetBoxIndices( nRow, nCol, nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax );
		
			uint32 nValuesUsed = MAX_SET_VALUE;
			uint32 nEmptyRow = 99;
			uint32 nEmptyCol = 99;

			for ( uint32 nBoxRow = nBoxRowMin; nBoxRow < nBoxRowMax; ++nBoxRow )
			{
				for ( uint32 nBoxCol = nBoxColMin; nBoxCol < nBoxColMax; ++nBoxCol )
				{
					const val_t nValue = m_Board[nBoxRow][nBoxCol];
					if ( 0 == nValue )
					{
						nEmptyRow = nBoxRow;
						nEmptyCol = nBoxCol;
					}
					else
					{
						nValuesUsed -= nValue;
					}
				}
			}

			if ( IsValueValid( nValuesUsed, nEmptyRow, nCol ) )
			{
				SetCellValue( nValuesUsed, nEmptyRow, nEmptyCol ); 
			}
			else
			{
				// board is a dud
				m_bPooched = true;
			}
		}
	}

	// Did we set any new values? If so, then we need to run this logic again, 
	// as there may be new holes to fill...
	if ( m_nScore != nOldScore && !IsSolved() )
	{
		FindTrivialSolutions();
	}


	return m_nScore;
}


uint32 Sudoku_Board::FindTrivialSolutions(const uint32 nRow, const uint32 nCol)
{
	const uint32 nOldScore = m_nScore;

	if ( 8 == GetRowCount(nRow) )
	{
		uint32 nValuesUsed = MAX_SET_VALUE;
		uint32 nEmptyCol = 99;

		for ( uint32 nCol = 0; nCol < NUM_COLS; nCol++ )
		{
			const val_t nValue = m_Board[nRow][nCol];
			if ( nValue == 0 )
				nEmptyCol = nCol;
			else
				nValuesUsed -= nValue;
		}

		if ( IsValueValid( nValuesUsed, nRow, nEmptyCol ) )
		{
			SetCellValue( nValuesUsed, nRow, nEmptyCol ); 
		}
		else
		{
			// board is a dud
			m_bPooched = true;
		}
	}


	if ( 8 == GetColCount(nCol) )
	{
		uint32 nValuesUsed = MAX_SET_VALUE;
		uint32 nEmptyRow = 99;

		for ( uint32 nRow = 0; nRow < NUM_ROWS; nRow++ )
		{
			const val_t nValue = m_Board[nRow][nCol];
			if ( nValue == 0 )
				nEmptyRow = nRow;
			else
				nValuesUsed -= nValue;
		}

		if ( IsValueValid( nValuesUsed, nEmptyRow, nCol ) )
		{
			SetCellValue( nValuesUsed, nEmptyRow, nCol );	
		}
		else
		{
			// board is a dud
			m_bPooched = true;
		}
	}

	if ( 8 == GetBoxCount( nRow, nCol ) )
	{
		uint32 nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax;	
		GetBoxIndices( nRow, nCol, nBoxRowMin, nBoxRowMax, nBoxColMin, nBoxColMax );

		uint32 nValuesUsed = MAX_SET_VALUE;
		uint32 nEmptyRow = 99;
		uint32 nEmptyCol = 99;

		for ( uint32 nBoxRow = nBoxRowMin; nBoxRow < nBoxRowMax; ++nBoxRow )
		{
			for ( uint32 nBoxCol = nBoxColMin; nBoxCol < nBoxColMax; ++nBoxCol )
			{
				const val_t nValue = m_Board[nBoxRow][nBoxCol];
				if ( 0 == nValue )
				{
					nEmptyRow = nBoxRow;
					nEmptyCol = nBoxCol;
				}
				else
				{
					nValuesUsed -= nValue;
				}
			}
		}

		if ( IsValueValid( nValuesUsed, nEmptyRow, nCol ) )
		{
			SetCellValue( nValuesUsed, nEmptyRow, nEmptyCol ); 
		}
		else
		{
			// board is a dud
			m_bPooched = true;
		}
	}

	// Did we set any new values? If so, then we need to run this logic again, 
	// as there may be new holes to fill...
	if ( m_nScore != nOldScore && !IsSolved() )
	{
		FindTrivialSolutions();
	}


	return m_nScore;
}


bool Sudoku_Board::CheckValidity() const
{
	for ( uint32 nRow = 0; nRow < NUM_ROWS; nRow++ )
	{
		uint32 nRowCount = 0;
		for ( uint32 nCol0 = 0; nCol0 < NUM_COLS; nCol0++ )
		{	
			const uint32 nBit = 1 << nCol0;
			const bool bHasBit = ((m_RowCache[nRow] & OCCUPANT_INDICES_MASK) & nBit) != 0;
			const bool bShouldHaveBit = m_Board[nRow][nCol0] >  0;

			// check validity of index cache
			if ( bHasBit != bShouldHaveBit )
			{
				DebugMessage( "Board row %d cache is messed up: %d != %d for %d (%d)\n", 
					nRow, bHasBit, bShouldHaveBit, nCol0, m_RowCache[nRow] & OCCUPANT_INDICES_MASK  );
				return false;
			}

			if ( m_Board[nRow][nCol0] == 0 )
				continue;

			nRowCount++;

			for ( uint32 nCol1 = 0; nCol1 < NUM_COLS; nCol1++ )
			{
				if ( nCol0 == nCol1 )
					continue;

				if ( m_Board[nRow][nCol0] == m_Board[nRow][nCol1] )
				{
					DebugMessage( "Board has duplicate value %d in column %d and %d of row %d\n", 
						m_Board[nRow][nCol0], nCol0, nCol1, nRow );
					return false;
				}
				
			}
		}

		// check validity of count cache
		if ( nRowCount != GetRowCount(nRow) )
		{
			DebugMessage( "Row %d count is %d, should be %d\n", 
				nRow, GetRowCount(nRow), nRowCount );
			return false;
		}
	}

	for ( uint32 nCol = 0; nCol < NUM_COLS; nCol++ )
	{
		uint32 nColCount = 0;
		for ( uint32 nRow0 = 0; nRow0 < NUM_ROWS; nRow0++ )
		{
			const uint32 nBit = 1 << nRow0;
			const bool bHasBit = ((m_ColCache[nCol] & OCCUPANT_INDICES_MASK) & nBit) != 0;
			const bool bShouldHaveBit = m_Board[nRow0][nCol] >  0;

			// check validity of index cache
			if ( bHasBit != bShouldHaveBit )
			{
				DebugMessage( "Board col %d cache is messed up: %d != %d for %d (%d)\n", 
					nCol, bHasBit, bShouldHaveBit, nRow0, m_ColCache[nCol] & OCCUPANT_INDICES_MASK  );
				return false;
			}

			if ( m_Board[nRow0][nCol] == 0 )
				continue;

			nColCount++;

			for ( uint32 nRow1 = 0; nRow1 < NUM_ROWS; nRow1++ )
			{
				if ( nRow0 == nRow1 )
					continue;

				if ( m_Board[nRow0][nCol] == m_Board[nRow1][nCol] )
				{
					DebugMessage( "Board has duplicate value %d in row %d and %d of column %d\n", 
						m_Board[nRow0][nCol], nRow0, nRow1, nCol );
					return false;
				}
				
			}
		}

		// check validity of count cache
		if ( nColCount != GetColCount(nCol) )
		{
			DebugMessage( "Col %d count is %d, should be %d\n", 
				nCol, GetColCount(nCol), nColCount );
			return false;
		}
	}

	uint32 nValuesUsed[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	for ( uint32 nRow = 0; nRow < NUM_ROWS; nRow++ )
	{
		for ( uint32 nCol = 0; nCol < NUM_COLS; nCol++ )
		{
			nValuesUsed[ m_Board[nRow][nCol] ]++;
		}
	}

	for ( uint32 i = 1; i < 10; i++ )
	{
		if ( 9 != (m_nValueCache[i] + nValuesUsed[i]) )
		{
			DebugMessage( "Value %d has been used %d times, but we have %d remaining\n", 
				nValuesUsed[i], m_nValueCache[i] );
			return false;
		}
	}

	return true;
}

class Sudoku_Solver
{
public:
	Sudoku_Solver();

	static const uint32 MAX_BOARDS = 1000;
	static const uint32 MAX_SOLUTION_STEPS = MAX_BOARDS * 10; 
	//static const uint32 MAX_SOLUTION_STEPS = 0xFFFFFFFF; 

	void Reset();
	bool CheckValidity() const;

	void Initialize(const char * problem);
	bool Solve();
	bool SolveBruteForce();
	bool GetSolution( char * solution );
	uint32 GetHighestScoredBoard( char * board );

	bool HasSolution() const;
	uint32 GetNumSteps() const;
	uint32 GetNumBoardsUsed() const;

	

private:

	inline Sudoku_Board	*AllocBoard();
	bool Strategy_Random( Sudoku_Board *pBoard );
	bool Strategy_BruteForce( Sudoku_Board *pBoard );
	bool ApplyStrategies( Sudoku_Board *pBoard );
	void DoSolutionStep();

	bool Strategy_BruteForceStep( Sudoku_Board *pBoard, uint32 nStartRow, uint32 nStartCol );

	Sudoku_Board	m_BoardData[MAX_BOARDS];	// board data used by Alloc(), don't use directly.. 
	Sudoku_Board	*m_BoardList[MAX_BOARDS];	// list of active boards

	uint32 m_nBoardCount;
	uint32 m_nSolutionSteps;

	bool m_bFoundSolution;

	Random m_Random;
};

Sudoku_Solver::Sudoku_Solver()
{
	memset( m_BoardData, 0, sizeof(m_BoardData) );
	memset( m_BoardList, 0, sizeof(m_BoardList) );

	m_nBoardCount = 0;
	m_nSolutionSteps = 0;

	m_bFoundSolution = false;
}

inline Sudoku_Board	*Sudoku_Solver::AllocBoard()
{
	FO_REALZ( m_nBoardCount < MAX_BOARDS, 
		"Sudoku_Solver ran out of boards!" );

	if ( m_nBoardCount >= MAX_BOARDS )
		return NULL;

	// grab the next available board
	Sudoku_Board *pNewBoard = &m_BoardData[m_nBoardCount];

	// get it ready for (re)use
	pNewBoard->Reset();

	// Toss it at the end of the active list for now 
	m_BoardList[m_nBoardCount] = pNewBoard; 	

	++m_nBoardCount;

	return pNewBoard;
}

bool Sudoku_Solver::HasSolution() const
{
	return m_bFoundSolution;
}

uint32 Sudoku_Solver::GetNumSteps() const
{
	return m_nSolutionSteps;
}

uint32 Sudoku_Solver::GetNumBoardsUsed() const
{
	return m_nBoardCount;
}

void Sudoku_Solver::Reset()
{
	m_nBoardCount = 0;
	m_nSolutionSteps = 0;
	m_bFoundSolution = false;
}

bool Sudoku_Solver::CheckValidity() const
{
	for ( uint32 nBoard = 0; nBoard < m_nBoardCount; nBoard++ )
	{
		if ( !m_BoardList[nBoard]->CheckValidity() )
			return false;

		if ( nBoard > 1 )
		{
			if ( m_BoardList[nBoard-1]->GetScore() > m_BoardList[nBoard]->GetScore() )
			{
				DebugMessage( "Boards at position %d and %d are out of order, scores are %d and %d\n", 
					nBoard-1, nBoard,
					m_BoardList[nBoard-1]->GetScore(), 
					m_BoardList[nBoard]->GetScore() );
		
				return false;
			}
		}
	}

	return true;
}

void Sudoku_Solver::Initialize(const char * problem)
{
	Reset();

	Sudoku_Board *pFirstBoard = AllocBoard();

	for ( uint32 r = 0; r < NUM_ROWS; ++r )
	{
		for ( uint32 c = 0; c < NUM_COLS; ++c )
		{
			const val_t v = problem[c+(r*9)];
			if ( 0 != v )
			{
				pFirstBoard->SetCellValue( v, r, c );
			}
		}
	}

	pFirstBoard->FindTrivialSolutions();	
	m_bFoundSolution = pFirstBoard->IsSolved();
}

bool Sudoku_Solver::Strategy_Random( Sudoku_Board *pBoard )
{
	// Totally, stupidly random. 
	sint32 random_tries = 1000;
	while( random_tries > 0 )
	{
		--random_tries;

		const uint32 nRow = m_Random.GetInt(8);
		const uint32 nCol = m_Random.GetInt(8);
		const val_t nVal = m_Random.GetInt(8)+1;

		if ( pBoard->IsValueValid( nVal, nRow, nCol ) )
		{
			// Set the value
			pBoard->SetCellValue( nVal, nRow, nCol );

			// Fill in any trivial values
			pBoard->FindTrivialSolutions();
			return true;
		}
	}



	return false;
}

bool Sudoku_Solver::Strategy_BruteForce( Sudoku_Board *pBoard )
{
	if ( Strategy_BruteForceStep(pBoard, 0, 0 ) )
	{
		return true;
	}

	return false;
}

bool Sudoku_Solver::Strategy_BruteForceStep( Sudoku_Board *pBoard, uint32 nStartRow, uint32 nStartCol )
{
	if ( pBoard->IsSolved() )
		return true;

	for ( uint32 nRow = nStartRow; nRow < NUM_ROWS; ++nRow )
	{
		if ( pBoard->IsRowFull(nRow) )
			continue;

		for ( uint32 nCol = nStartCol; nCol < NUM_COLS; ++nCol )
		{
			if ( pBoard->IsColFull(nCol) )
				continue;

			if ( pBoard->IsCellOccupied(nRow, nCol) )
				continue;

			for ( val_t nValue = 1; nValue <= 9; ++nValue )
			{
				if ( pBoard->IsValueValid(nValue, nRow, nCol) )
				{
					pBoard->SetCellValue(nValue, nRow, nCol);

					pBoard->FindTrivialSolutions(nRow, nCol);	

					if ( !pBoard->IsUnsolveable() )
					{
						if ( Strategy_BruteForceStep( pBoard, nRow, nCol ) )
						{
							return true;
						}
					}

					pBoard->ClearCellValue( nRow, nCol );
				}

			}
		}

	}

	return false;
}

bool Sudoku_Solver::ApplyStrategies( Sudoku_Board *pBoard )
{
	if ( Strategy_Random(pBoard) )
	{
		return true;
	}

	return false;
}

void Sudoku_Solver::DoSolutionStep()
{
	FO_REALZ( m_nBoardCount > 0, 
		"DoSolutionStep called when solver is not intialized!" );

	// Find the range of the highest scored boards
	const uint32 nLastIdx = m_nBoardCount-1;
	uint32 nFirstIdx = 0;


	/**/
	uint32 nBestScores = m_BoardList[0]->GetScore();

	for ( uint32 nIdx = 1; nIdx < nLastIdx; ++nIdx )
	{
		const uint32 nThisScore = m_BoardList[nIdx]->GetScore();
		if ( nThisScore > nBestScores )
		{
			nBestScores = nThisScore;
			nFirstIdx = nIdx;
		}
	}
	


	// Randomly select a board from that range
	const uint32 nTestIdx = nFirstIdx != nLastIdx ? m_Random.GetInt( nFirstIdx, nLastIdx ) : nFirstIdx;

	FO_REALZ( nTestIdx < m_nBoardCount, "Board selection out of range!" );


	// Create a copy of the board to store a new board state
	Sudoku_Board *pNewBoard = AllocBoard();
	const uint32 nBoardAllocIdx = m_nBoardCount-1;
	pNewBoard->CopyFrom( m_BoardList[ nTestIdx ] );


	// Find a new valid board based on that board state
	if ( ApplyStrategies( pNewBoard ) )
	{
#if BROCK_DEBUG && 1
		char board[100]; memset( board, 0, sizeof(board) );
		uint32 nScore = GetHighestScoredBoard(board);
		DebugMessage( "\nNew Board %d/%d Step %d/%d Scored: %d\n", m_nBoardCount, MAX_BOARDS, m_nSolutionSteps, MAX_SOLUTION_STEPS, nScore );
		Generator::print(board);
#endif

		// Is that new board solved? If so, we're done. OMG!
		if ( pNewBoard->IsSolved() )
		{
			m_bFoundSolution = true;
		}
		else if ( pNewBoard->IsUnsolveable() )
		{
			// the board is in an illogical state from bad guesses. 
			// recycle it 
			--m_nBoardCount;
			m_BoardList[ nBoardAllocIdx ] = NULL;
		}
		else
		{
			// Otherwise, insert it into the list at an index based on it's score. 
			// (Note that alloc puts it at the end of the list)
			const uint32 nNewScore = pNewBoard->GetScore();			
			for ( uint32 nIdx = 1; nIdx < m_nBoardCount; ++nIdx )
			{
				const uint32 nCurrScore = m_BoardList[ nIdx ]->GetScore();
				if ( nNewScore < nCurrScore )
				{
					// Found a board with a higher score... 

					// shift everything down. 
					for ( uint32 nMoveIdx = m_nBoardCount-1; nMoveIdx > nIdx; --nMoveIdx )
					{
						m_BoardList[ nMoveIdx ] = m_BoardList[ nMoveIdx-1 ];
					}

					// insert it here
					m_BoardList[ nIdx ] = pNewBoard;
					break;
				}

			}
		}
	}
	else
	{
		// For some reason, we couldn't fill in any squares(????)
		// For now, just return the board to the depot
		--m_nBoardCount;
		m_BoardList[ nBoardAllocIdx ] = NULL;
	}


	PARANOID( CheckValidity(), "Solver Invalid in DoSolutionStep!\n" );
}

bool Sudoku_Solver::Solve()
{
	// stop after a lot of tries, so we don't loop in here forever!	

	uint32 nHighestCount = 0;
	uint32 nHighestScore = 0;
	do 
	{
		DoSolutionStep();
		
		++m_nSolutionSteps;

		if ( m_nBoardCount > nHighestCount ||
			 m_BoardList[ m_nBoardCount-1 ]->GetScore() > nHighestScore )
		{
			nHighestCount = m_nBoardCount;
			nHighestScore = m_BoardList[ m_nBoardCount-1 ]->GetScore();

			DebugMessage( "Solver Board %u/%u Step %u/%u Scored: %d\n", m_nBoardCount, MAX_BOARDS, m_nSolutionSteps, MAX_SOLUTION_STEPS, nHighestScore );
		}

	}
	while ( !m_bFoundSolution && m_nBoardCount < MAX_BOARDS-1 && m_nSolutionSteps < MAX_SOLUTION_STEPS );

	return m_bFoundSolution;
}

bool Sudoku_Solver::SolveBruteForce()
{
	Sudoku_Board *pBoard = m_BoardList[ 0 ];

	FO_REALZ( CheckValidity(), "Solver is in invalid state" );

	Strategy_BruteForce( pBoard );
	m_bFoundSolution = pBoard->IsSolved();

	return m_bFoundSolution;
}

bool Sudoku_Solver::GetSolution( char * solution )
{
	if ( !m_bFoundSolution )
		return false;

	FO_REALZ( CheckValidity(), "Solver is in invalid state" );
	FO_REALZ( m_nBoardCount > 0, "Asked to get solution when there are no boards generated" );

	// The solved solution is always the last solution in the active list. 
	Sudoku_Board *pBoard = m_BoardList[ m_nBoardCount-1 ];

	FO_REALZ( pBoard->IsSolved(), "Board list is f'ed up" );

	pBoard->ToCharArray(solution);

	return true;
}

uint32 Sudoku_Solver::GetHighestScoredBoard( char * board )
{
	//FO_REALZ( CheckValidity(), "Solver is in invalid state" );
	FO_REALZ( m_nBoardCount > 0, "Asked to get solution when there are no boards generated" );

	// The solved solution is always the last solution in the active list. 
	Sudoku_Board *pBoard = m_BoardList[ m_nBoardCount-1 ];

	pBoard->ToCharArray(board);

	return pBoard->GetScore();
}

namespace Brock
{
	static Sudoku_Solver s_Solver;
	static bool s_bInitialized = false;
	void initialize()
	{
		if ( s_bInitialized ) 
			return;
		s_bInitialized = true;

#if BROCK_DEBUG || BROCK_PROFILE
		DebugMessage( "Size of Sudoku_Board = %d bytes (%.3f kb)\n", 
			sizeof(Sudoku_Board), sizeof(Sudoku_Board) / 1024.0f );

		DebugMessage( "Size of Sudoku_Solver = %d bytes (%.3f kb)\n", 
			sizeof(Sudoku_Solver), sizeof(Sudoku_Solver) / 1024.0f );
#endif

	}

	void find_solution( const char * problem, char * solution )
	{
		// run some once-only initialization
		initialize();

#if BROCK_PROFILE
		Timer SolutionTimer;
		SolutionTimer.Start();
#endif 

		// Intialize the solver
		s_Solver.Initialize(problem);

#if BROCK_DEBUG
		char board[100]; 
		memset( board, 0, sizeof(board) );
		uint32 nScore = s_Solver.GetHighestScoredBoard(board);
		DebugMessage( "Board intial score: %d\n", nScore );
		Generator::print2(problem, board);
#elif BROCK_PROFILE
		char board[100]; 
		memset( board, 0, sizeof(board) );
		uint32 nScore = s_Solver.GetHighestScoredBoard(board);
		DebugMessage( "Starting new board, initial score: %d\n", nScore );
#endif

		// Try to find a solution
		//if (s_Solver.Solve())
		//{
		//	s_Solver.GetSolution(solution);		
		//}

		if (s_Solver.SolveBruteForce())
		{
			s_Solver.GetSolution(solution);		
		}


		FO_REALZ( s_Solver.CheckValidity(), "Solver finished in invalid state!\n" );


#if BROCK_PROFILE		
		if (s_Solver.HasSolution())
		{
			DebugMessage( "Found Puzzle Solution in %d steps, %d boards:\n",
				s_Solver.GetNumSteps(), s_Solver.GetNumBoardsUsed() );
			Generator::print2(problem, solution);
		}
		else
		{
			char board[100]; 
			memset( board, 0, sizeof(board) );
			uint32 nScore = s_Solver.GetHighestScoredBoard(board);

			DebugMessage( "Could Not Solve puzzle in %d steps, %d boards, highest score %d:\n",
				s_Solver.GetNumSteps(), s_Solver.GetNumBoardsUsed(), nScore );
			Generator::print2(problem, board);
		}

		float64 fElapsed = SolutionTimer.Elapsed();
		if ( fElapsed > 60.0f )
		{
			DebugMessage( "Solution took %12.6f seconds (%12.3f mins)\n",
				fElapsed,
				fElapsed / 60.0f );
		}
		else if ( fElapsed < 1.0f )
		{
			DebugMessage( "Solution took %12.6f seconds (%12.3f msec)\n",
				fElapsed,
				fElapsed * 1000.0f );
		}
		else
		{
			DebugMessage( "Solution took %12.6f seconds (%12.3f msec)\n", fElapsed );
		}
#endif 
		
	}
}